@@ -241,7 +241,7 @@ test_that("noPredictions.asreml42", {
                          args=list(fixed = y ~ Species*Substrate*Irrigation,
                                    random = ~ Row + Column,
                                    keep.order=TRUE, data = gw.dat, 
-                                   maxiter=50, workspace = 1e08, stepsize = 0.0001))
+                                   maxit=50, workspace = 1e08, stepsize = 0.0001))
   current.asrt <- as.asrtests(current.asr, NULL, NULL)
   current.asrt <- rmboundary(current.asrt)
   testthat::expect_error(diffs <- predictPresent(current.asrt$asreml.obj,
